///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to "CELayoutEditor.cpp".
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

#ifndef _EDITOR_DOCUMENT_H_
#define _EDITOR_DOCUMENT_H_

// Standard wxWidget includes
#include "wx/wx.h"
// More specific wxWidgets includes
//#include "wx/glcanvas.h"
#include "wx/docview.h"
#include "wx/hashmap.h"
//#include "PropertiesPanel.h"
//#include "wx/cmdproc.h"
// Maybe someday replace by wxWidgets containers(?)
//#include <map>
//#include <vector>
//#include <set>
//#include "IDocumentObserver.h"
//#include "Selection.h"
//#include "CopiedSelection.h"

/** The application's Document implementation.
*
* This class is used as interface between the application and the CEGUI system. It is important to know that this
* class is like a "thin client". It does not cach anything (except for a few pointers). Each call is made directly
* on CEGUI, so no seperate administrations have to be maintained, which makes this class remarkeable easy to understand
* I think.
*
*/

class PropertiesPanel;

class EditorDocument : public wxDocument
{
    DECLARE_DYNAMIC_CLASS(EditorDocument)

    // Attributes
public:
    /** Constructor.
    */
    EditorDocument();

    /** Opens the given layout file as editable.
    */
    bool OnOpenDocument(const wxString& filename);

    /** Starts a new layout.
    */
    bool OnNewDocument() ;

    /** Saves the current layout to its filepath.
    */
    bool OnSaveDocument(const wxString& filename);

    /** The current layout has been closed. The system has prompted the user to save changes, if any.
    */
    bool OnCloseDocument();

    /** Sets the document to being 'dirty' and updates the attached views.*/
    void Modify (bool modify);

    /** Resets the document for editing eg. after a resolution-change which re-starts Ogre.*/
    void reset () ;

    // possible actions on the document 
    void HandleChangeImageFile(const wxString& newfilename);
    void HandleChangeImagesetName(const wxString& newname);
    void HandleChangeNativeRes(const wxPoint& newnativeres);
    void HandleChangeAutoScaled(const bool newAutoScaled);
    void HandleAddRegion(const wxString &name, const wxRect &dim);
    void HandleDelRegion(const wxString &name);
    void HandleDelAllRegion();
    void HandleMoveScaleRegion(const wxString &name, const wxRect &dim);
    void HandleRenameRegion(const wxString &currentname, const wxString &newname);


    void setPropertiesPanel(PropertiesPanel* PropsPanel){m_panelProperties= PropsPanel;};
    const wxString &getImagesetName() const {return m_nameImageset;}
private:

    PropertiesPanel* m_panelProperties;
    /*************************************************************************
    Implementation Constants
    *************************************************************************/
    static const wxPoint defaultNativeRes;				//!< default value for the NativeRes value.

    // Document data
    wxString m_filenameXml;

    // Document data
    wxString m_nameImageset;

    // Document data
    wxString m_filenameImageset;

    // Document data
    wxPoint m_ptNativeRes;

    // Document data
    bool m_bAutoScaled;

    // Document data
    WX_DECLARE_STRING_HASH_MAP( wxRect, mapNamedRegion );

    mapNamedRegion  m_mapSetRectangles;

};

#endif // _EDITOR_DOCUMENT_H_
