/////////////////////////////////////////////////////////////////////////////
// Name:        ImagesetEditor.h
// Purpose:     wxBitmap class
// Author:      Julian Smart
// Modified by:
// Created:     01/02/97
// RCS-ID:      $Id: pngdemo.h,v 1.7 2004/10/06 20:54:09 ABX Exp $
// Copyright:   (c) Julian Smart
// Licence:     wxWindows license
/////////////////////////////////////////////////////////////////////////////

#ifndef _PROPERTIES_PANEL_H_
#define _PROPERTIES_PANEL_H_

#if defined( __GNUG__) && !defined(__APPLE__)
#pragma interface
#endif

// Include wxWidgets' headers

#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include "EditorDocument.h"

#include <wx/grid.h>
#include <wx/dynarray.h>


class EditorView;
// Define a new canvas which can receive some events
class PropertiesPanel: public wxScrolledWindow
{
public:
    PropertiesPanel(wxWindow *parent, const wxPoint& pos, const wxSize& size);
    ~PropertiesPanel(void){};

    void setView (EditorView* aView)
    {
        view = aView;
    }



    void createContent();
    void reset();
    //void setImageFilename();
    void OnLoadImageFile(wxCommandEvent& WXUNUSED(event));
    void OnGridAddRow(wxCommandEvent& WXUNUSED(event));
    void OnGridDelRow(wxCommandEvent& WXUNUSED(event));
    void OnGridCellChange(wxGridEvent& event);
    void OnGridCellEdit(wxGridEvent& event);
    void OnGridCellSelect(wxGridEvent& event);

    wxString getImagesetName();
    void setImagesetName(const wxString &name);

    wxString getImageFileName();
    void setImageFileName(const wxString &name);

    wxString getXmlFileName();
    void setXmlFileName(const wxString &name);

    wxPoint getNativeRes();
    void setNativeRes(const wxPoint &NativeRes);

    bool getAutoScaled();
    void setAutoScaled(const bool AutoScaled);

    int selectRegion(const wxString &name);
    void addRegion(const wxString &name, const wxRect &dim, int rowindex = -1);
    void movescaleRegion(const wxString &name, const wxRect &dim);
    wxRect getRegion(const wxString &name);
    int delRegion(const wxString &name);
    void delAllRegion();

    bool getSelected(wxString &name)
    {     
        if (m_PropsGrid && 
            m_PropsGrid->GetNumberRows() > 0)
        { 
            wxArrayInt sel = m_PropsGrid->GetSelectedRows();
            if (sel.Count() > 0)
            {
                const size_t rowindex = sel.Item(0);
                name = m_PropsGrid->GetCellValue(rowindex, 0);
                return true;
            }
        }
        return false;
    }

private:
    size_t mUniqueGuid;
    static const char charParseNativeRes;
    static const wxString EmmitterPropertiesPanel;

    void createPropsGrid(wxWindow *parent);

    /** The view to which we are attached.*/
    EditorView *view;

    // PropertiesPanel widgets
    wxTextCtrl      *m_txtImagesetName;
    wxTextCtrl      *m_txtImageFileName;
    //wxTextCtrl      *m_txtXmlFileName;
    wxButton        *m_btnLoadImage;
    wxGrid          *m_PropsGrid;
    wxComboBox      *m_cbboxNativeRes;
    wxCheckBox      *m_ckboxAutoScaled;

    wxString m_oldRowName;

    DECLARE_EVENT_TABLE()
};

const int ID_TEXT = 10000;
const int ID_TEXTCTRL = 10001;
const int ID_BUTTON = 10002;
const int ISE_TEXTCTRL_IMAGE = 10003;
const int ISE_BUTTON_LOAD_IMAGE = 10004;
const int ID_COMBO = 10005;
const int ID_CHECKBOX = 10006;
const int ID_GRID = 10007;
const int ID_SCROLLED = 10008;
const int ID_GRID_ADD_ROW = 10009;
const int ID_GRID_DEL_ROW = 10010;

#endif // _PROPERTIES_PANEL_H_
