///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to "CELayoutEditor.cpp".
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

#ifdef __linux__
# include <unistd.h>

# define DATAPATH_VAR_NAME "CELAYOUTEDITOR_DATAPATH"
#endif


#include <wx/dir.h>

#include "EditorFrame.h"
#include "EditorGLCanvas.h"
// Utility class for convertion between CEGUI- and wxWidgets strings
#include "StringHelper.h"

// Toolbar icons
#include "bitmaps/new.xpm"
#include "bitmaps/open.xpm"
#include "bitmaps/save.xpm"
#include "bitmaps/cut.xpm"
#include "bitmaps/copy.xpm"
#include "bitmaps/paste.xpm"
#include "bitmaps/delete.xpm"
#include "bitmaps/help.xpm"

#define ASSERT_DOCUMENT_MSG "The m_document member is NULL!"

using namespace CEGUI;

IMPLEMENT_CLASS(EditorFrame, wxDocParentFrame)

//-----------------------------------------------------------------------
BEGIN_EVENT_TABLE(EditorFrame, wxDocParentFrame)
//		EVT_TEXT(ISE_TEXTCTRL_IMAGE, EditorFrame::OnLoadImageFile)
      EVT_MENU(ISE_MENU_BACKGROUND_COLOR, EditorFrame::OnChangeBackgroundColor)
END_EVENT_TABLE()

//-----------------------------------------------------------------------
EditorFrame::EditorFrame(wxDocManager *manager, wxFrame *frame, wxWindowID id, const wxString& title,
                 const wxPoint& pos, const wxSize& size, const long type):
wxDocParentFrame(manager, frame, id, title, pos, size, type)
{
  //
  m_document = (EditorDocument*) NULL;
  
	//m_GUIRenderer = new OpenGLRenderer (0);
 // m_GUISystem = new System(m_GUIRenderer);
  m_glcanvasImageset = 0;
  m_PropsPanel = 0;

  // Add a menu bar, a toolbar and a statusbar
  AttachMenubar();
  AttachToolbar();
  CreateStatusBar(2);

  // Create the main dialog
	createContent();
}

//-----------------------------------------------------------------------
EditorFrame::~EditorFrame()
{
}

//-----------------------------------------------------------------------
void EditorFrame::AttachMenubar()
{
		wxMenuBar *menu_bar = new wxMenuBar;

	wxMenu* menu_file = new wxMenu;
	menu_file->Append( wxID_NEW, wxT("New\tCtrl-N"), wxT("Create a new document.") );
	menu_file->Append( wxID_OPEN, wxT("Load...\tCtrl-L"), wxT("Open an existing document.") );
	menu_file->Append( wxID_SAVEAS, wxT("Save...\tCtrl-S"), wxT("save the active document.") );
  menu_file->Append(wxID_CLOSE, wxT("&Close \tCtrl-K"), wxT("Close the active document."));
	menu_file->AppendSeparator();
	menu_file->Append( wxID_EXIT, wxT("Quit\tCtrl-Q"), wxT("Quit the application; prompts to save document.") );
	menu_bar->Append( menu_file, wxT("File") );

	wxMenu* menu_options = new wxMenu;
	//menu_options->Append( ISE_LIST_TRACK_IMAGE, wxT("Image track list selection"), wxT("Image track list selection.") );
	//menu_options->Append( ISE_IMAGE_TRACK_LIST, wxT("List track image selection"), wxT("List track image selection.") );
	//menu_options->AppendSeparator();
	menu_options->Append( ISE_MENU_BACKGROUND_COLOR, wxT("Set Background color..."), wxT("choice the color of the background.") );
	//menu_options->Append( ISE_MENU_REGION_COLOR, wxT("Set Region color..."), wxT("choice the color of the region bounding box.") );
	//menu_options->Append( ISE_MENU_REGION_HIGHLIGHT_SCOLOR, wxT("Set Region selected color..."), wxT("choice the color of the selected region bounding box.") );
	menu_bar->Append( menu_options, wxT("Options") );

	wxMenu* menu_help = new wxMenu;
	menu_help->Append( wxID_HELP, wxT("Help..."), wxT("") );
	menu_help->Append( wxID_ABOUT, wxT("About..."), wxT("Display program information, version number and copyright.") );
	menu_bar->Append( menu_help, wxT("?") );

	SetMenuBar( menu_bar );

}

//-----------------------------------------------------------------------
void EditorFrame::AttachToolbar()
{
  // Create toolbar with icons-only
  wxToolBar* toolBar = new wxToolBar(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTB_HORIZONTAL); // |wxNO_BORDER
  //
  toolBar->AddTool(wxID_NEW, wxBitmap (new_xpm));
  toolBar->AddTool(wxID_OPEN, wxBitmap (open_xpm));
  toolBar->AddTool(wxID_SAVE, wxBitmap (save_xpm));
  toolBar->AddSeparator();
  toolBar->AddTool(wxID_CUT, wxBitmap (cut_xpm));
  toolBar->AddTool(wxID_COPY, wxBitmap (copy_xpm));
  toolBar->AddTool(wxID_PASTE, wxBitmap (paste_xpm));
  toolBar->AddTool(wxID_CLEAR, wxBitmap (delete_xpm));
  toolBar->AddSeparator();
  toolBar->AddTool(wxID_ABOUT, wxBitmap (help_xpm));
  // Don't forget this one!
  toolBar->Realize();
  SetToolBar(toolBar);

}

//-----------------------------------------------------------------------
void EditorFrame::SetDocument(EditorDocument* document)
{
  m_document = document;
 
  // Always set the (new) document
	if ( m_document && m_PropsPanel)
	{
        if(m_PropsPanel)
            m_document->setPropertiesPanel(m_PropsPanel);
	  // Tell the document's command processor where it can find the Undo and Redo items
		//m_document->GetCommandProcessor()->SetEditMenu(mEditMenu);
	}
}

void EditorFrame::createContent()
{
  
    m_splitter = new wxSplitterWindow(this, ISE_SPLITTER_FRAME, wxDefaultPosition, wxDefaultSize,wxSP_NOBORDER);   
    m_splitter->SetSashGravity(0.0);

	// spitter child, left part
	m_PropsPanel = new PropertiesPanel(m_splitter, wxDefaultPosition, wxSize(300,100));
  //m_PropsPanel->Show(false);
	m_PropsPanel->createContent();
  m_PropsPanel->Disable();

}

//-----------------------------------------------------------------------
EditorGLCanvas *EditorFrame::AttachCanvas(EditorView *view)
{
   
    int width, height;

    // Create the GLCanvas. This apprauch provides MDI support maybe in the future
    EditorGLCanvas* canvas = new EditorGLCanvas(view, m_splitter, wxDefaultPosition, wxDefaultSize);
 	// add the canvas to the frame
    // Tell it about the initial size
    m_splitter->SplitVertically(m_PropsPanel, canvas, 250);
    canvas->GetClientSize(&width, &height);
    canvas->changeSize(width, height);
    canvas->ClearBackground();
    canvas->Disable();

    // Copy into member
    m_glcanvasImageset = canvas;

    return canvas;
}

//-----------------------------------------------------------------------
void EditorFrame::setCanvasSize(int newWidth, int newHeight)
{
  m_glcanvasImageset->changeSize( newWidth, newHeight);
}

void EditorFrame::OnChangeBackgroundColor(wxCommandEvent& WXUNUSED(event))
{
  m_glcanvasImageset->changeBackgroundColor();

}