///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to "CELayoutEditor.cpp".
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////
#include <wx/colordlg.h>

#include "EditorGLCanvas.h"
#include "PropertiesPanel.h"
#include "EditorView.h"
#include "StringHelper.h"

using namespace CEGUI;
//using namespace std;

//-----------------------------------------------------------------------
BEGIN_EVENT_TABLE(EditorGLCanvas, wxGLCanvas)
EVT_SIZE            (EditorGLCanvas::OnResize)
EVT_PAINT			    	(EditorGLCanvas::OnPaint)
EVT_ERASE_BACKGROUND(EditorGLCanvas::OnErase)
EVT_MOTION				  (EditorGLCanvas::OnMouseMotion)
EVT_LEFT_DOWN			  (EditorGLCanvas::OnLeftDown)
EVT_LEFT_UP			  	(EditorGLCanvas::OnLeftUp)
EVT_RIGHT_DOWN		  (EditorGLCanvas::OnRightDown)
EVT_RIGHT_UP			 	(EditorGLCanvas::OnRightUp)
EVT_MOUSEWHEEL      (EditorGLCanvas::OnMouseWheel)
EVT_KEY_DOWN		  	(EditorGLCanvas::OnKeyDown)
EVT_KEY_UP			  	(EditorGLCanvas::OnKeyUp)
END_EVENT_TABLE()

const wxString EditorGLCanvas::EmmitterGLCanvas( "EmmitterGLCanvas" );

//-----------------------------------------------------------------------
EditorGLCanvas::EditorGLCanvas(EditorView *v, wxWindow* parent, const wxPoint& pos, const wxSize& size):
wxGLCanvas(parent, (wxGLCanvas*) NULL, -1, pos, size , wxSUNKEN_BORDER ) // | wxVSCROLL | wxHSCROLL )
{
    view = v;
    mDraggedWindow = 0;
    m_bUseBigSteps = true;
    m_bMovingAll = false;
    m_bMove = false;
    m_bScale = true;
    m_offsetX = 0.0f;
    m_offsetY = 0.0f;
    m_FactorZoom = 1.0f;
    m_GUIRenderer = new OpenGLRenderer (0);
    m_GUISystem = new System(m_GUIRenderer);
    m_BackgroundColor.Set(255, 255, 255);

    // Init one-time-only stuff
    SetCurrent();

    glShadeModel(GL_SMOOTH);
    glEnable(GL_DEPTH_TEST);

    // Set the clear color to a grey value
    glClearColor (0.0, 0.0, 0.0, 1.0);

    // Depth buffer setup
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);


    // Create the CEGUI renderable
    InitializeCEGUI();
}

void EditorGLCanvas::setImage(const wxString &name)
{
    // TODO : mettre un try / catch pour gerer si imageset foireuse ou deja existante

    // delete the old one
    if (ImagesetManager::getSingleton().isImagesetPresent ("CEImagesetEditor"))
        ImagesetManager::getSingleton().destroyImageset("CEImagesetEditor");

    mRealWidth = 0;
    mRealHeight = 0;

    // load the new one
    // TODO : trace the load fail and inform the user
    if(!name.IsEmpty())
    {
        const Image *img = m_imagesetStaticImage->getImage ();
        if (img == 0)  
        {
            Imageset* imageset = ImagesetManager::getSingleton().createImagesetFromImageFile(
                "CEImagesetEditor", 
                StringHelper::ToCEGUIString(name) );
            imageset->setAutoScalingEnabled(false);
            assert (imageset->isImageDefined ("full_image")) ; 
            m_imagesetStaticImage->setImage("CEImagesetEditor", "full_image");
            img = m_imagesetStaticImage->getImage ();
        }

        Size sizeImage = img->getSize();
        mRealWidth =  sizeImage.d_width;
        mRealHeight =  sizeImage.d_height;
        computZoomFactor(0);
    }
    else
    {
        m_imagesetStaticImage->setSize(Size(mRealWidth, mRealWidth));
    }

    Render();
}

wxString EditorGLCanvas::getImage()
{
    return "toto";
}
//-----------------------------------------------------------------------  
void EditorGLCanvas::changeSize(int newWidth, int newHeight)
{
    // Tell the canvas!
    //SetSize(newWidth, newHeight);
    //GetParent()->SetSize(newWidth, newHeight);
    //GetGrandParent()->SetVirtualSize(newWidth, newHeight);

    //mCurrentWidth = newWidth;
    //mCurrentHeight = newHeight;

    // Tell openGL
    glViewport(0, 0, (GLsizei)newWidth, (GLsizei)newHeight);

    // Calculate aspect ratio
    gluPerspective(45.0f,(GLfloat)newWidth/(GLfloat)newHeight,0.1f,100.0f);
    m_GUIRenderer->setDisplaySize( CEGUI::Size((float)GetClientSize().x, (float)GetClientSize().y));


}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnResize(wxSizeEvent& event) 
{
    // We need the client size
    int clientWidth, clientHeight;
    GetClientSize(&clientWidth, &clientHeight);

    changeSize(clientWidth, clientHeight);
}

//-----------------------------------------------------------------------
void EditorGLCanvas::InitializeCEGUI()
{
    //m_GUIRenderer = new OpenGLRenderer (0);
    //m_GUISystem = new System(m_GUIRenderer);

    // Load the scheme to initialse the Marti Skin which we use in this sample
    SchemeManager::getSingleton().loadScheme("../datafiles/schemes/Marti.scheme");
    //FontManager::getSingleton().createFont("../datafiles/fonts/arial-12.font");

    Logger::getSingleton().setLoggingLevel (Informative);

    resetCEGUI();
}

//-----------------------------------------------------------------------
bool EditorGLCanvas::resetCEGUI()
{

    // Get window manager which we wil use for a few jobs here.
    WindowManager& winMgr = WindowManager::getSingleton();

    WindowFactoryManager& wfMgr = WindowFactoryManager::getSingleton();

    // Cleanup any current windows to avoid name clashes
    winMgr.destroyAllWindows ();

    try 
    {
        StaticImage* m_root = static_cast<StaticImage*>(winMgr.createWindow( "Marti/StaticImage", "Root"));
        // setActiveLayout (root);
        // setting
        colour rootBackColor( 0.6f, 0.6f, 0.6f );
        m_root->setAreaRect( Rect(0, 0, 1, 1));
        m_root->setBackgroundColours( rootBackColor);
        m_root->setFrameEnabled(false);

        m_imagesetStaticImage = static_cast<StaticImage*>(winMgr.createWindow( "Marti/StaticImage", "Image"));
        m_imagesetStaticImage->setWindowPosition(UVector2(UDim( 0.0f, 0.0f), UDim( 0.0f, 0.0f)));
        m_imagesetStaticImage->setWindowSize(UVector2(UDim( 0.0f, 512.0f), UDim( 0.0f, 512.0f)));
        m_imagesetStaticImage->setMaximumSize(Size(20480.0, 20480.0));
        m_imagesetStaticImage->setClippedByParent(false);

        m_imagesetStaticImage->setBackgroundColours( StringHelper::WXColorToCEGUIColor(m_BackgroundColor));
        m_root->addChildWindow(m_imagesetStaticImage);


        /*
        // Since we have overwritten this method, we must notify the views ourselves!
        //UpdateAllViews();

        SetFilename(filename, true);
        SetTitle(filename);
        */

        m_GUISystem->setGUISheet(m_root);
        return true;
    }
    catch (Exception& guiException) 
    {
        // Cleanup so far to avoid name clashes for correctly loaded windows
        WindowManager::getSingletonPtr()->destroyAllWindows ();
        wxLogError (wxT("%s"), StringHelper::ToWXString(guiException.getMessage()).c_str());
        return false;
    }
}

void EditorGLCanvas::selectRegion(const wxString &name)
{   
    if(m_imagesetStaticImage->isChild(StringHelper::ToCEGUIString(name)))
    {
        Window* selectedWin =  m_imagesetStaticImage->getChild(StringHelper::ToCEGUIString(name));
        CEGUI::WindowManager::WindowIterator it = CEGUI::WindowManager::getSingleton ().getIterator();
        while (!it.isAtEnd())
        {
            //CEGUI::String name  = it.getCurrentKey();
            CEGUI::Window *win= it.getCurrentValue();
            if (selectedWin != win && win->isActive())
                win->deactivate();
            ++it;
        }
        if (!selectedWin->isActive())
        {
            selectedWin->activate();
            Render();
        }
    }  
}
void EditorGLCanvas::renameRegion(const wxString &currentname, const wxString &newname, const wxRect &dim)
{
    if(m_imagesetStaticImage->isChild(StringHelper::ToCEGUIString(currentname)))
    {
        FrameWindow* toolbox = static_cast<FrameWindow*>(m_imagesetStaticImage->getChild(StringHelper::ToCEGUIString(currentname)));        
        toolbox->destroy ();
        addRegion (newname, dim);
        selectRegion(newname);
    }
}

//-----------------------------------------------------------------------
void EditorGLCanvas::addRegion(const wxString &name, const wxRect &dim)
{
    FrameWindow* toolbox = static_cast<FrameWindow*>
        (WindowManager::getSingleton().createWindow("Marti/FrameWindow", StringHelper::ToCEGUIString(name)));

    toolbox->setTitleBarEnabled(false);
    //toolbox->setSizingEnabled(false);
    toolbox->setCloseButtonEnabled(false);

    float posx = static_cast<float>(dim.GetX()) / mRealWidth;
    float posy = static_cast<float>(dim.GetY()) / mRealHeight;
    float width = static_cast<float>(dim.GetWidth()) / mRealWidth;
    float height = static_cast<float>(dim.GetHeight()) / mRealHeight;
    toolbox->setWindowPosition(UVector2(UDim( posx, 0.0f), UDim( posy, 0.0f)));
    toolbox->setWindowSize(UVector2(UDim( width, 0.0f), UDim( height, 0.0f)));

    /*
    // Besion de injecttime pour marcher
    wxChar strDim [100];
    wxSnprintf (strDim, sizeof(strDim), wxT("\n[%d, %d] [%d, %d]"), dim.GetX(), dim.GetY(), dim.GetWidth(), dim.GetHeight()) ;
    String strTooltip = StringHelper::ToCEGUIString(name);
    strTooltip.append(strDim);
    toolbox->setTooltipText(strTooltip);
    */
    //toolbox->setMinimumSize(Size(1.0 / mRealWidth, 1.0/ mRealHeight));
    toolbox->setProperty( "Alpha", "0.25");
    toolbox->setProperty( "SizingBorderThickness", "2.0");

    //toolbox->subscribeEvent(FrameWindow::EventMouseEnters, CEGUI::Event::Subscriber(&EditorGLCanvas::handleRegionMouseEnter, this));
    //toolbox->subscribeEvent(FrameWindow::EventMouseLeaves, CEGUI::Event::Subscriber(&EditorGLCanvas::handleRegionMouseLeave, this));

    toolbox->subscribeEvent(FrameWindow::EventActivated, CEGUI::Event::Subscriber(&EditorGLCanvas::handleRegionMouseActivate, this));
    toolbox->subscribeEvent(FrameWindow::EventDeactivated, CEGUI::Event::Subscriber(&EditorGLCanvas::handleRegionMouseDeActivate, this));

    toolbox->subscribeEvent(FrameWindow::EventMouseMove, CEGUI::Event::Subscriber(&EditorGLCanvas::handleRegionModified, this));
    //toolbox->subscribeEvent(FrameWindow::EventSized, CEGUI::Event::Subscriber(&EditorGLCanvas::handleRegionModified, this));

    m_imagesetStaticImage->addChildWindow(toolbox);
    Render();

}


void EditorGLCanvas::movescaleRegion(const wxString &name, const wxRect &dim)
{
    if(m_imagesetStaticImage->isChild(StringHelper::ToCEGUIString(name)))
    {
        Window* win =  m_imagesetStaticImage->getChild(StringHelper::ToCEGUIString(name));

        float posx = static_cast<float>(dim.GetX()) / mRealWidth;
        float posy = static_cast<float>(dim.GetY()) / mRealHeight;
        float width = static_cast<float>(dim.GetWidth()) / mRealWidth;
        float height = static_cast<float>(dim.GetHeight()) / mRealHeight;
        win->setWindowPosition(UVector2(UDim( posx, 0.0f), UDim( posy, 0.0f)));
        win->setWindowSize(UVector2(UDim( width, 0.0f), UDim( height, 0.0f)));
        win->activate();

        Render();
    }  
}

wxRect EditorGLCanvas::getRegion(const wxString &name)
{
    wxRect dim;
    return dim;
}

void EditorGLCanvas::delRegion(const wxString &name)
{
    if(m_imagesetStaticImage->isChild(StringHelper::ToCEGUIString(name)))
    {
        m_imagesetStaticImage->getChild(StringHelper::ToCEGUIString(name))->destroy();
        Render();
    }
}

void EditorGLCanvas::delAllRegion()
{
    Render();
}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnMouseMotion(wxMouseEvent& event)
{
    //view->SetEventEmmiter(EmmitterGLCanvas);
    this->SetFocus();

    // tell CEGUI about the event
    System::getSingleton().injectMousePosition(event.GetX(), event.GetY());

    // Check this, because we might get this event during document closing
    if (view && view->GetDocument() && 
        (event.GetX() != mLastMouseX || event.GetY() != mLastMouseY))
    {
        mLastMouseX = event.GetX();
        mLastMouseY = event.GetY();
        if(event.Dragging() && event.RightIsDown() && mDraggedWindow)
        {     
            const float newXpos = mLastMouseX - m_offsetX;
            const float newYpos = mLastMouseY - m_offsetY;
            mDraggedWindow->setXPosition(MetricsMode::Absolute,  newXpos);
            mDraggedWindow->setYPosition(MetricsMode::Absolute,  newYpos);           
        }

        // m_offset prend pour l'instant sur imageset, et non sur la framewindow selectionne
        /*
        else if (event.Dragging() && event.LeftIsDown())
        {  

        }
        */
        // Update

        // updating statusbar with the mouse position on the imageset ( computing offset and zoom)
        int mousePosX = m_imagesetStaticImage->getXPosition(MetricsMode::Absolute);
        int mousePosY = m_imagesetStaticImage->getYPosition(MetricsMode::Absolute);
        mousePosX = (mLastMouseX - mousePosX ) / m_FactorZoom;
        mousePosY = (mLastMouseY - mousePosY ) / m_FactorZoom;
        view->updateMouseStatusBar(mousePosX, mousePosY);

        // updating statusbar with the and zoom and the imageset size
        view->updateDocStatusBar(m_FactorZoom, mRealWidth, mRealHeight );

        Render();
    }
}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnMouseWheel(wxMouseEvent& event)
{
    // Check this, because we might get this event during document closing
    if (view && view->GetDocument())
    {
        int MouseWheelDirection = event.GetWheelRotation();
        computZoomFactor(MouseWheelDirection);
        view->updateDocStatusBar(m_FactorZoom, mRealWidth, mRealHeight );
        Render();
    }

}
//-----------------------------------------------------------------------
void EditorGLCanvas::OnLeftDown(wxMouseEvent& event)
{     
    Window* win = System::getSingleton().getWindowContainingMouse();
    if( win->testClassName("FrameWindow") )
    {
        m_offsetX = event.GetX() - win->getXPosition(MetricsMode::Absolute);
        m_offsetY = event.GetY() - win->getYPosition(MetricsMode::Absolute);
        mDraggedWindow = win;
    }
    else
    {
        mDraggedWindow = 0;
    }


    // Check this, because we might get this event during document closing
    if (view && view->GetDocument())
    {
        // tell CEGUI about the event
        System::getSingleton().injectMouseButtonDown(MouseButton::LeftButton);
        Render();
    }
    event.Skip();
}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnLeftUp(wxMouseEvent& event)
{
    m_offsetX = 0;
    m_offsetY = 0;
    mDraggedWindow = 0;
    // Check this, because we might get this event during document closing
    if (view && view->GetDocument())
    { 
        // tell CEGUI about the event
        System::getSingleton().injectMouseButtonUp(MouseButton::LeftButton);
        Render();
    }	
    event.Skip();
}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnRightDown(wxMouseEvent& event)
{ 
    Window* win = System::getSingleton().getWindowContainingMouse();
    if( win->testClassName("FrameWindow") )
    {
        m_offsetX = event.GetX() - win->getXPosition(MetricsMode::Absolute);
        m_offsetY = event.GetY() - win->getYPosition(MetricsMode::Absolute);
        mDraggedWindow = win;
    }
    else if (m_bMove)
    {
        m_offsetX = event.GetX() - m_imagesetStaticImage->getXPosition(MetricsMode::Absolute);
        m_offsetY =event.GetY() - m_imagesetStaticImage->getYPosition(MetricsMode::Absolute);
        mDraggedWindow = m_imagesetStaticImage;
    }
    else
    {
        mDraggedWindow = 0;
    }

    // tell CEGUI about the event
    System::getSingleton().injectMouseButtonDown(MouseButton::RightButton);
    Render();

    event.Skip();

}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnRightUp(wxMouseEvent& event)
{
    m_offsetX = 0;
    m_offsetY = 0;
    mDraggedWindow = 0;

    // tell CEGUI about the event
    System::getSingleton().injectMouseButtonUp(MouseButton::RightButton);
    Render();
    /*	
    // Check this, because we might get this event during document closing
    if (view && view->GetDocument())
    {
    EditorDocument* document = (EditorDocument*) view->GetDocument();
    document->HandleMouseReleased () ;
    }
    */	
}


//-----------------------------------------------------------------------
void EditorGLCanvas::computZoomFactor(int MouseWheelDirection)
{

    // Logger::getSingleton().logEvent("imagesetSize '" + AppliConfigFile +"'.", Standard);
    if(MouseWheelDirection > 0)
        m_FactorZoom += 0.125;
    else if(MouseWheelDirection < 0)
        m_FactorZoom -= 0.125;


    // extrem down limit verif
    if (m_FactorZoom < 0.25)
        m_FactorZoom = 0.25;

    // extrem up limit verif
    if (m_FactorZoom > 10.0)
        m_FactorZoom = 10.0;

    float width = m_FactorZoom * mRealWidth;
    float heigh = m_FactorZoom * mRealHeight;
    m_imagesetStaticImage->setWidth(MetricsMode::Absolute, width);
    m_imagesetStaticImage->setHeight(MetricsMode::Absolute, heigh);
}

//-----------------------------------------------------------------------
bool EditorGLCanvas::Reset()
{

    return resetCEGUI();
}

//-----------------------------------------------------------------------
void EditorGLCanvas::Render()
{
    if (view && view->GetDocument())
    {
        SetCurrent();

        // clear color and depth buffers
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        // CEGUI rendering
        RenderLayout();		

        // Resizer rendering
        //DrawGrid();

        glFlush();    
        SwapBuffers();
    }
}

//-----------------------------------------------------------------------
void EditorGLCanvas::DrawGrid()
{
    //wxLogDebug(wxT("Painting resizers."));

    EditorDocument* document = (EditorDocument*) view->GetDocument();
    int i;
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(m_offsetX, m_offsetY, 0.0f);
    // Turn the lines GREEN
    glColor3f (1.0f, 0.0f, 0.0f);
    glDisable(GL_TEXTURE_2D);

    glBegin(GL_LINES);

    // Draw a 1x1 grid along the X and Z axis'
    for(i = 0; i <= 10; i += 1)
    {
        // Do the horizontal lines (along the X)
        glVertex2i(0, i); glVertex2i(10,i);

        // Do the vertical lines (along the Z)
        glVertex2i(i, 0); glVertex2i(i, 10);
    }
    glEnd();
    glEnable(GL_TEXTURE_2D);

    // wxLogDebug(wxT("Draw rectangle at %f, %f, %f, %f"), x0, y0, x1, y1);

}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnPaint( wxPaintEvent& event )
{
    // We need to do this to avoid a flood of paint events
    wxPaintDC paintDC(this);

    Render();
}

//-----------------------------------------------------------------------
void EditorGLCanvas::RenderLayout()
{
    if (CEGUI::System::getSingletonPtr() != NULL)
    {
        //wxLogDebug(wxT("Painting layout."));
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();

        // Render the current layout
        CEGUI::System::getSingleton().renderGUI();
    }

}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnErase(wxEraseEvent& event)
{
    // Draw set background.. in progress
}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnKeyDown(wxKeyEvent& event)
{

    if (event.AltDown ())
    {
        m_bMove = true;
        m_bScale = false;
    }
    else    
    {
        m_bMove = false;
        m_bScale = true;
    }
    /*
    int key = event.GetKeyCode();
    CSelectionMover::MoveDirection moveDirection = CSelectionMover::NotSet;

    // Control key is used to change selection by keys to small steps (when you're near your target location.)
    // Control key is used to change selection by keys to small steps (when you're near your target location.)
    if (event. ControlDown())
    {
    m_bUseBigSteps = false;
    }
    if (key == 'w' || key == 'W' || key == WXK_UP)
    {
    moveDirection = CSelectionMover::MoveUp;
    }
    else if (key == 's' || key == 'S' || key == WXK_DOWN)
    {
    moveDirection = CSelectionMover::MoveDown;
    }
    else if (key == 'a' || key == 'A' || key == WXK_LEFT)
    {
    moveDirection = CSelectionMover::MoveLeft;
    }
    else if (key == 'd' || key == 'D' || key == WXK_RIGHT)
    {
    moveDirection = CSelectionMover::MoveRight;
    }

    // Was a relevant selection mover key pressed?
    if (moveDirection != CSelectionMover::NotSet)
    {
    CSelectionMover(((EditorDocument*)view->GetDocument())->getSelection()).moveSelection(moveDirection, m_bUseBigSteps);
    view->GetDocument()->Modify(true);
    }
    */
}

//-----------------------------------------------------------------------
void EditorGLCanvas::OnKeyUp(wxKeyEvent& event)
{
    if (event.AltDown ())
    {
        m_bMove = true;
        m_bScale = false;
    }
    else    
    {
        m_bMove = false;
        m_bScale = true;
    }
    // Control key is used to change selection by keys to small steps (when you're near your target location.)
    if (event.ControlDown())
    {
        m_bUseBigSteps = true;
    }
}

void EditorGLCanvas::changeBackgroundColor()
{ 
    //ColourRect colorBack(m_imagesetStaticImage->getBackgroundColours());
    //colour bkgrd = colorBack.d_top_left;
    //m_BackgroundColor.Set(bkgrd.getRed(), 
    wxColourData colorBackground;
    colorBackground.SetColour(m_BackgroundColor);

    // Show color selector
    wxColourDialog dialog(this, &colorBackground);
    if (dialog.ShowModal() == wxID_OK)
    {
        colorBackground = dialog.GetColourData();
        m_BackgroundColor = colorBackground.GetColour();
        // colorBack.
        if(m_imagesetStaticImage)
        {
            // argb_t
            colour bkgrd;
            bkgrd.setRed( m_BackgroundColor.Red()/255.0);
            bkgrd.setGreen( m_BackgroundColor.Green()/255.0);
            bkgrd.setBlue( m_BackgroundColor.Blue()/255.0);
            m_imagesetStaticImage->setBackgroundColours(StringHelper::WXColorToCEGUIColor(m_BackgroundColor));
            Render();
        }
    }

}

bool EditorGLCanvas::handleRegionMouseActivate(const CEGUI::EventArgs& e)
{
    using namespace CEGUI;

    // Check this, because we might get this event during document closing
    if ( view && view->GetDocument() )
    {

        FrameWindow* win = static_cast<FrameWindow*> (System::getSingleton().getWindowContainingMouse());
        if(win)
        {
            //win->setProperty( "Alpha", "0.65");
            //win->activate();

            wxString name = StringHelper::ToWXString(win->getName());
            view->getPropsPanel()->selectRegion(name);
        }   
    }

    return true;
}

bool EditorGLCanvas::handleRegionMouseDeActivate(const CEGUI::EventArgs& e)
{
    using namespace CEGUI;

    FrameWindow* win = static_cast<FrameWindow*> (System::getSingleton().getWindowContainingMouse());
    if (win)
    {
        //win->setProperty( "Alpha", "0.25");
        //win->deactivate();
    }
    //e.window->deactivate();
    // event was handled
    return true;
}
/*
bool EditorGLCanvas::handleRegionMouseEnter(const EventArgs& e)
{
using namespace CEGUI;

// Check this, because we might get this event during document closing
if ( view && view->GetDocument() )
{

FrameWindow* win = static_cast<FrameWindow*> (System::getSingleton().getWindowContainingMouse());
if(win)
{
win->setProperty( "Alpha", "0.65");
win->activate();

wxString name = StringHelper::ToWXString(win->getName());
view->getPropsPanel()->selectRegion(name);
}   
}

return true;
}

bool EditorGLCanvas::handleRegionMouseLeave(const EventArgs& e)
{
using namespace CEGUI;

FrameWindow* win = static_cast<FrameWindow*> (System::getSingleton().getWindowContainingMouse());
if (win)
{
win->setProperty( "Alpha", "0.25");
win->deactivate();
}
//e.window->deactivate();
// event was handled
return true;
}
*/
// TODO : pour l'instant, appel par mousemouve, a n'appeller que quand il y a changement de taille ou deplacement.
bool EditorGLCanvas::handleRegionModified(const EventArgs& e)
{
    using namespace CEGUI;
    //MouseButton btn = ((MouseEventArgs)e).button;
    //(btn == MouseButton::LeftButton) && 

    // Check this, because we might get this event during document closing
    if ( view && view->GetDocument() )
    {

        Window* win = System::getSingleton().getWindowContainingMouse();
        if(win)
        {
            wxString name = StringHelper::ToWXString(win->getName());
            float posx = win->getXPosition(MetricsMode::Relative) * mRealWidth;
            float posy = win->getYPosition(MetricsMode::Relative) * mRealHeight;
            float width = win->getWidth(MetricsMode::Relative) * mRealWidth;
            float height = win->getHeight(MetricsMode::Relative) * mRealHeight;

            EditorDocument* document = (EditorDocument*) view->GetDocument();
            document->HandleMoveScaleRegion(name, wxRect(posx, posy, width, height));
            view->getPropsPanel()->movescaleRegion(name, wxRect(posx, posy, width, height));
        }   
    }

    // event was handled
    return true;
}


//virtual void  onMouseMove (MouseEventArgs &e) 
